import processing.serial.*;

Serial myPort;    // The serial port:
PFont myFont;     // The display font:
int proto_num = 4;  //number of prototypes - this parameter needs to be adjusted to match the PIC
int dimensions = 2; //number of dimensions in data - this parameter needs to be adjusted to match the PIC
int[] inBuffer = new int[10];  //buffer to store incoming serial data
int [][] protos = new int[proto_num][dimensions]; //array to store most recent prototype values
int [][] protos_last = new int[proto_num][dimensions]; //array to store last prototype values
color [] proto_colors = new color[proto_num]; //array of colors, 1 to represent each prototype 
int bufferIndex=0;
PrintWriter output; 
int in = 0;
int [] sensors = new int[dimensions];
int dataIndex=0;
int init_counter=0;
byte phase_flag = 0;
int iteration = 50;  //offset for graph

void setup() {
  textAlign(LEFT, TOP);
  int i,j;
  size(450,450);
  myFont = loadFont("CourierNew36.vlw");
  textFont(myFont, 13); 
  myPort = new Serial(this, Serial.list()[0], 9600); //cretae the serial port, COM1 9600 baud
  output = createWriter("capture.txt"); //save sserial data to capture.txt
  background(255);   //set window background to white
  stroke(0);     // Set line drawing color to black
  fill(0);
  text("time -> ",170,325);
  line(50,310,350,310);
  line(350,310,350,50);
  text("255", 358,300); 
  text("value", 358,150); 
  text("0",358,50); 
  fill(255,0,0);
  stroke(255,0,0);
  rect(60,360,10,10);
  text("temperature", 100, 360); 
  fill(0,0,255);
  stroke(0,0,255);
  rect(240,360,10,10);
  text("light", 280, 360); 
  //initialize proto colors
  for (i=0; i<proto_num; i++){
    proto_colors[i] = color(int(random(0,225)),int(random(0,225)),int(random(0,225)));
  }
}

void draw() {
  int i, j;

  if (myPort.available()>0){
    //if data available, read a byte
    in = myPort.read();
    println(in);//print the byte
    inBuffer[bufferIndex] = in; //add byte to the buffer
    //if last added was cr (decimal 10) check out what value it is 
    //is it 1 of 2 sensor values or a prototype value
    if ((in == 10)&& (bufferIndex>=3)){
      //a message was received
      // bufferIndex = 10, carriage return
      //bufferIndex-1 = value sensor or proto
      //bufferIndex-2 = dimension, x or y (0 or 1)
      //bufferIndex -3 = s for sensor values or a number for what proto it is
      //bufferIndex-4 = p for proto values
      println("buffer index: "+bufferIndex);
      println(inBuffer);
      if (inBuffer[bufferIndex-3] == 115) { //s -- sensor data received
        //assign data to array
        sensors[inBuffer[bufferIndex-2]] = inBuffer[bufferIndex-1];
        //write data to file
        output.println("sensor "+ inBuffer[bufferIndex-2]+": "+inBuffer[bufferIndex-1]+"\r\n");
        if (inBuffer[bufferIndex-2]==1){
          //if y coordinate rec. draw data point
          if (phase_flag == 1){
            stroke(0);
            fill(0);
            ellipse(sensors[0]+50,sensors[1]+50,5,5);

            fill(255);
            stroke(255);
            rect(0,360,width,height-360);
            fill(255,0,0);
            stroke(255,0,0);
            text("temperature: "+sensors[0], 100, 360); 
            fill(0,0,255);
            stroke(0,0,255);
            text("light: "+sensors[1], 280, 360); 


            fill(proto_colors[0]);
            stroke(proto_colors[0]);
            text("p0: "+protos[0][0]+", "+protos[0][1], 5, 400);
            fill(proto_colors[1]);
            stroke(proto_colors[1]);
            text("p1: "+protos[1][0]+", "+protos[1][1], 115, 400);
            fill(proto_colors[2]);
            stroke(proto_colors[2]);
            text("p2: "+protos[2][0]+", "+protos[2][1], 225, 400);
            fill(proto_colors[3]);
            stroke(proto_colors[3]);
            text("p3: "+protos[3][0]+", "+protos[3][1], 335, 400);
          }
          if (phase_flag == 0){
            //just plot the data point
            stroke(255,0,0);
            fill(255,0,0);
            ellipse(iteration,sensors[0]+50,3,3);
            stroke(0,0,255);
            fill(0,0,255);
            ellipse(iteration,sensors[1]+50,3,3);
            iteration+=6;
            if (iteration>300) iteration = 0;
          }
        }
        bufferIndex=0; //re-init buffer
      }

      else if ((bufferIndex>3)&&(inBuffer[bufferIndex-4] == 112)){ //p
        if (phase_flag==0) {//if we missed the phase flag
          phase_flag = 1;
          save("data.png"); //save image from display window
          stroke(255);
          fill(255);
          rect(0,0,400,400);  //wipe display clean for next phase

          //draw gridlines and text
          stroke(0);     // Set line drawing color to black
          fill(0);
          text("0",50,325); 
          text("temperature",170,325);
          line(50,310,350,310);
          line(350,310,350,50);
          text("255", 358,300); 
          text("light", 358,150); 
          text("0",358,50); 
        }
        //proto value rec.
        //assign values to proto array:
        //assign current proto to last
        protos_last[inBuffer[bufferIndex-3]][inBuffer[bufferIndex-2]]=protos[inBuffer[bufferIndex-3]][inBuffer[bufferIndex-2]];
        protos[inBuffer[bufferIndex-3]][inBuffer[bufferIndex-2]]=inBuffer[bufferIndex-1];
        if (inBuffer[bufferIndex-2]==1){
          //if y coordinate rec. draw data point
          stroke(255,120);
          fill(255,120);
          rect(protos_last[inBuffer[bufferIndex-3]][0]+50,protos_last[inBuffer[bufferIndex-3]][1]+50,5,5);
          stroke(proto_colors[inBuffer[bufferIndex-3]]);
          fill(proto_colors[inBuffer[bufferIndex-3]]);
          rect(protos[inBuffer[bufferIndex-3]][0]+50,protos[inBuffer[bufferIndex-3]][1]+50,5,5);

          //write data to file
          output.println("proto "+ inBuffer[bufferIndex-3]+": "+protos[inBuffer[bufferIndex-3]][0]+", "+protos[inBuffer[bufferIndex-3]][1]+"\r\n");
        }

        bufferIndex=0; //re-init buffer
      }
      else if ((inBuffer[bufferIndex-3] == 102)&&(inBuffer[bufferIndex-2] == 102)&&(inBuffer[bufferIndex-1] == 102)) {
        phase_flag = 1;
        //save image of graph
        save("data.png"); //save image from display window
        stroke(255);
        fill(255);
        rect(0,0,400,400);  //wipe display clean for next phase

        //draw gridlines and text
        stroke(0);     // Set line drawing color to black
        fill(0);
        text("0",50,325); 
        text("temperature",170,325);
        line(50,310,350,310);
        line(350,310,350,50);
        text("255", 358,300); 
        text("light", 358,150); 
        text("0",358,50); 
      }
      else bufferIndex++; //if not a sensor message and not a proto message just continue
    } //end if 10/byte3+

      else bufferIndex++;
    if (bufferIndex == 10) bufferIndex=0;//if we hit the end of our buffer wrap around
  } //end if byte available

} //end draw



void keyPressed(){ // Press e key to save the data
  if (key == 'e'){
    output.flush(); // Write the remaining data
    output.close(); // Finish the file
    save("SOM.png"); //save image from display window
    exit(); // Stop the program
  }
}

