//code for a 4 prototype 2 dimensional SOM

#include "C:\Program Files\PICC\projects\kohonen SOM\som1.h"
#include <MATH.H>
#define RAND_MAX 50  // specify the range of random integers to generate (0  50)
#include <STDLIB.H>
#define proto_num 4  //number prototypes
#define dimensions 2  //number of dimensions of the data
#define samples	48	  //how many samples of data to collect
#define iterations 10 //how many times to iterate through learning cycle
#define LED PIN_D3
#define output0 PIN_C0
#define output1 PIN_C1
#define output2 PIN_C2
#define output3 PIN_C3


long round(float x){
 long x2;
 if (x>=0){
	x2 = (long)(x+0.5);
 }
 else {
	x2 = (long)(x-0.5);
}
 return x2;
}

float sqr(float x){
   return x * x;
}

float distance(int x1, int y1, int x2, int y2){
   return sqrt( sqr((float)x1-(float)x2) + sqr((float)y1-(float)y2));
}

void main() {
   int i,j,k,count;
   int location = 0;
   int value,x,y;
   int prototypes[proto_num][dimensions];
   float activation[proto_num];
   int winner;
   float c = .025;
   signed long temp1;
   float temp2;
   float meanX, meanY;

   setup_adc_ports(RA0_RA1_RA3_ANALOG);
   setup_adc(ADC_CLOCK_INTERNAL);
   setup_psp(PSP_DISABLED);
   setup_spi(SPI_SS_DISABLED);
   setup_counters(RTCC_INTERNAL,RTCC_DIV_2);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);

   output_low(output0);
   output_low(output1);
   output_low(output2);
   output_low(output3);

   output_high(LED);
   delay_ms(500);
   output_low(LED);
   delay_ms(500);
   output_high(LED);
   delay_ms(500);
   output_low(LED);
   delay_ms(500);
   output_high(LED);
   //first off, collect data
   //collect 48 samples, 1 every 30 minutes, write to eeprom
   i=0;
   while (i<samples){

      //read sensor 1
      set_adc_channel(0);
      delay_us(10);
      x = read_adc();
      delay_ms(10);
      //read sensor 2
      set_adc_channel(1);
      delay_us(10);
      y = read_adc();
      delay_ms(10);

      //store data
      write_eeprom(location, x);
      location++;
      delay_ms(10);
      write_eeprom(location, y);
      location++;
      delay_ms(10);
      //send data to host
      output_low(LED);
      putc('s');
	   putc(0);
	   putc(x);
	   putc(10);

      putc('s');
	   putc(1);
	   putc(y);
	   putc(10);
	  putc(i); //print out count
      output_high(LED);
      i++;
     
      for (k=0;k<30;k++){     //delay 15 minutes (12 hr. total cycle)
         for (j=0;j<60;j++){  //delay 1 minute
            delay_ms(1000);  //delay 1 sec.
         }
      }
   }

   //now we have 96 addresses in eeprom representing 48 2-d datapoints
   //calculate the mean for both sensors
   meanX = 0;

   for (location=0;location<(samples*2);location+=2){
      x = read_eeprom(location);
      meanX+= x;
      delay_ms(10);
   }
   meanX /= (float) samples;
   meanY = 0;
   for (location=1;location<(samples*2);location+=2){
      y = read_eeprom(location);
      meanY+= y;
      delay_ms(10);
   }
   meanY /= (float) samples;

   //printf("meanX: %f, meanY: %f\r\n", meanX, meanY);

   for (i=0;i<proto_num;i++){ 
     prototypes[i][0] = rand()+round((meanX-25)); 
	 prototypes[i][1] = rand()+round((meanY-25)); 
   }
   //PHASE 2
   output_low(LED);
   output_high(output0);
   output_high(output1);
   output_high(output2);
   output_high(output3);
   putc(102);
   putc(102);
   putc(102);
   putc(10);
   delay_ms(100);
   //now learn from data
   for (count=0;count<iterations;count++){
      for (location=0;location<(samples*2);location+=2){
		 delay_ms(1000); //for readability
         x = read_eeprom(location);
         putc('s');
         putc(0);
         putc(x);
         putc(10);
         delay_ms(10);
         y = read_eeprom(location+1);
         putc('s');
         putc(1);
         putc(y);
         putc(10);
         delay_ms(10);
         //now find closest prototype to datapoint
         for (i=0;i<proto_num;i++){
            activation[i] = distance(x,y,prototypes[i][0],prototypes[i][1]);
         }
         //find the winner
         winner = 0;
         for (i=1;i<proto_num;i++){
            if (activation[i] < activation[winner]) winner = i;
         }
         
         //update winning proto; 2d data
         
         temp1 = (long)x - (long)prototypes[winner][0];
         temp2 = c*temp1;
         prototypes[winner][0] = round( prototypes[winner][0]+ temp2);

		 temp1 = (long)y - (long)prototypes[winner][1];
         temp2 = c*temp1;
         prototypes[winner][1] = round( prototypes[winner][1]+ temp2);
         
         //print out list of all prototype values
         for (i=0;i<proto_num;i++){
            for (j=0;j<dimensions;j++){
               putc('p');
               putc(i);
               putc(j);
               putc(prototypes[i][j]);
               putc(10);
            }
         }
      }
	  //could subtract from learning rate here
   }
   output_low(output0);
   output_low(output1);
   output_low(output2);
   output_low(output3);
   output_high(LED);

   //now continue updating dynamically with a reduced learning rate
   c = .005;
   while (true){
      output_high(LED);
      //read sensor 1
      set_adc_channel(0);
      delay_us(10);
      x = read_adc();
      delay_ms(10);
	  putc('s');
	  putc(0);
	  putc(x);
	  putc(10);

      //read sensor 2
      set_adc_channel(1);
      delay_us(10);
      y = read_adc();
	  putc('s');
	  putc(1);
	  putc(y);
	  putc(10);

	  //now find closest prototype to datapoint
     for (i=0;i<proto_num;i++){
         activation[i] = distance(x,y,prototypes[i][0],prototypes[i][1]);
     }

     //find the winner
     winner = 0;
     for (i=1;i<proto_num;i++){
      if (activation[i] < activation[winner]) winner = i;
     }

	 //output winner
     switch (winner){
	 case 0:
 		output_low(output1);
	  	output_low(output2);
		output_low(output3);
     	output_high(output0);
		break;
     case 1:
		output_low(output0);
	  	output_low(output2);
		output_low(output3);
     	output_high(output1);
		break;
	 case 2:
		output_low(output0);
	  	output_low(output1);
		output_low(output3);
     	output_high(output2);
		break;
	  case 3:
		output_low(output0);
	  	output_low(output1);
		output_low(output2);
     	output_high(output3);
		break;
     }
     /*
	 //update winning proto
	 for (i=0;i<dimensions;i++){
		temp1 = (long)x - (long)prototypes[winner][i];
	    temp2 = c*temp1;
		prototypes[winner][i] = round( prototypes[winner][i]+ temp2);
	 }

	 //print out list of all prototype values
	 for (i=0;i<proto_num;i++){
     	for (j=0;j<dimensions;j++){
			putc('p');
	        putc(i);
	 	    putc(j);
     	    putc(prototypes[i][j]);
	 	    putc(10);
		}
   	 }
    */
    output_high(LED);
	delay_ms(1000); //for readability
	
   }
}
